using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;
using System.IO;
using EnvDTE;
using SVNCOM;

namespace SVN_NET
{
	/// <summary>
	/// Summary description for UpdateForm.
	/// </summary>
	public class UpdateForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.TextBox outputBox;
		private System.Windows.Forms.Button okButton;

		private string updatedir;
		private SVNCOM.SVN svn;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public UpdateForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.outputBox = new System.Windows.Forms.TextBox();
			this.okButton = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// outputBox
			// 
			this.outputBox.AcceptsReturn = true;
			this.outputBox.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.outputBox.CausesValidation = false;
			this.outputBox.Multiline = true;
			this.outputBox.Name = "outputBox";
			this.outputBox.ReadOnly = true;
			this.outputBox.ScrollBars = System.Windows.Forms.ScrollBars.Both;
			this.outputBox.Size = new System.Drawing.Size(464, 136);
			this.outputBox.TabIndex = 10;
			this.outputBox.Text = "";
			this.outputBox.WordWrap = false;
			// 
			// okButton
			// 
			this.okButton.Location = new System.Drawing.Point(384, 144);
			this.okButton.Name = "okButton";
			this.okButton.TabIndex = 11;
			this.okButton.Text = "OK";
			this.okButton.Click += new System.EventHandler(this.okButton_Click);
			// 
			// UpdateForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(464, 174);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.okButton,
																		  this.outputBox});
			this.Name = "UpdateForm";
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "UpdateForm";
			this.Load += new System.EventHandler(this.UpdateForm_Load);
			this.ResumeLayout(false);

		}
		#endregion

		private void okButton_Click(object sender, System.EventArgs e)
		{
			this.DialogResult = DialogResult.OK;
			this.Close();
			return;
		}

		private void UpdateThread()
		{
			svn = new SVNCOM.SVN();
			SVNCOM._ISVNEvents_NotifyEventHandler notifyeventobj = new SVNCOM._ISVNEvents_NotifyEventHandler(this.Notify);
			svn.Notify += notifyeventobj;
			SVNCOM._ISVNEvents_PromptEventHandler prompteventobj = new SVNCOM._ISVNEvents_PromptEventHandler(this.Prompt);
			svn.Prompt += prompteventobj;
			try
			{
				Invoke(new UpdateDelegate(SetMessage), new object [] {"start update"});
				string[] str = updatedir.Split(new char[] {';'});
				foreach (string dir in str)
				{
					svn.update(dir, -1, true);
				}
				Invoke(new UpdateDelegate(SetMessage), new object [] {"finished."});
				Invoke(new UpdateDelegate(SetMessage), new object [] {"SVN_END"});
			}
			catch (ThreadAbortException)
			{
				Invoke(new UpdateDelegate(SetMessage), new object [] {"Exception"});
			}
			catch (System.Exception ex)
			{
				System.Windows.Forms.MessageBox.Show(ex.Message,"Update");
				Invoke(new UpdateDelegate(SetMessage), new object [] {"Exception"});
			}
			svn.Prompt -= prompteventobj;
			svn.Notify -= notifyeventobj;
		}

		private void Notify(string path, notify_action action, node_kind kind, string mime_type, notify_state content, notify_state prop, int revision)
		{
			if (revision < 1)
			{
				string str;
				notify_state _enum = ((int)content < (int)prop) ? content : prop;
				switch (_enum)
				{
					case SVNCOM.notify_state.notify_state_conflicted:
						str = "conflict!    ";
						break;
					case SVNCOM.notify_state.notify_state_inapplicable:
						str = "inapplicable ";
						break;
					case SVNCOM.notify_state.notify_state_merged:
						str = "merged       ";
						break;
					case SVNCOM.notify_state.notify_state_modified:
						str = "modified     ";
						break;
					case SVNCOM.notify_state.notify_state_unchanged:
						str = "unchanged    ";
						break;
					case SVNCOM.notify_state.notify_state_unknown:
						str = "updated      ";
						break;
					default:
						str = "             ";
						break;
				}
				str = str + path;
				Invoke(new UpdateDelegate(SetMessage), new object [] {str});
			}
			else
				Invoke(new UpdateDelegate(SetMessage), new object [] {"Revision : "+revision.ToString()});
		}

		private void SetMessage(string str)
		{
			if (str.IndexOf("Exception")>=0)
			{
				this.okButton.Enabled = true;
				return;
			}
			if (str.IndexOf("SVN_END")>=0)
			{
				this.okButton.Enabled = true;
				return;
			}
			outputBox.AppendText(str);
			outputBox.AppendText(Environment.NewLine);
		}

		private void Prompt(out string info, string prompt, bool hide)
		{
			PromptForm pf = new PromptForm();
			pf.SetValues(prompt,hide);
			info="";
			if (pf.ShowDialog()==System.Windows.Forms.DialogResult.OK)
			{
				info = pf.GetInfo();
			}
			else
				info = "";
		}

		public void SetUpdateDir(string str)
		{
			updatedir = str;
		}

		private void UpdateForm_Load(object sender, System.EventArgs e)
		{
			Debug.WriteLine("thread started");
			this.okButton.Enabled = false;
			System.Threading.Thread t = new System.Threading.Thread(new ThreadStart(UpdateThread));
			t.IsBackground = true;
			t.Name = "Update";
			t.Start();	
		}

	}
	public delegate void UpdateDelegate(string message);

}
